title: VIS IWS TM HK Monitor
author: M.Whillock@ucl.ac.uk
date: 23 Apr 2020

Copyright © 2020 University College London / MSSL

--
# Table of contents
- [Introduction](#introduction)
- [Prerequisites](#prerequisites)
- [Installation](#installation)
	- [Online Deployment](#online)
	- [Offline Deployment](#offline)
- [TM HK Monitor](#monitor)
	- [Launching](#launching)
	- [Operation](#operation)
	- [Testing](#testing)
	- [Future Work](#future)


# Introduction <a name="introduction"></a>


This a mini user guide to the VIS IWS TM HK Monitor program.

## Prerequisites <a name="prerequisites"></a>

* git
* python3
* conda
* pyqtgraph

## Installation <a name="installation"></a>

### Online Deployment <a name="online"></a>

TBD

### Offline Deployment <a name="offline"></a>

Choose a location to untar the HKTM_monitor gzipped tar file and untar it.

`tar xvfz HKTM_monitor.tar.gz`


## TM HK Monitor <a name="monitor"></a>

This program is used to display housekeeping parameters received in real-time from the SCOE or other sources.

### Launching <a name="launching"></a>

Ensure that the correct conda environment is activated:

`conda activate IWSop`

Change directory to where the program is installed.
This document assumes that python3 is run using the `python` command.
Use whatever command is required to run python3 on your system.

Launch the program:

`python ./iws_tmhk_monitor.py`

This will bring up a dialog box in order to select a display configuration:

![configuration](Pictures/dialog.png)

A configuration contains information about which housekeeping parameters are to be displayed and the names of the tabs in which they appear.
Configurations will be defined using the parameter chooser program.

Select a configuration and press the **Ok** button.

The main window will appear:

![main_window](Pictures/HK_monitor.png)


The following picture shows the different areas of the main window:

![main_window_bits](Pictures/HK_monitor_sections.png)

- Control Buttons
This area contains the buttons to start and stop updating and the quit program button.
- Text Displays
This area contains the text output boxes.
The upper text box displays information about events received.
The lower text box displays information about the telemetry packets received.
- Graph Tabs
This area consists of a tab widget with tabs for displaying different parameters and associated plots.
Each parameter has a coloured box next to it's label showing the colour of the line plotted in the graph.
- Parameter Display Tabs
This area is a tab widget with tabs displaying different HK parameters.
The tab names are based on the configuration choice made at the program start up.

The unmarked bottom area contains a status bar at the left end with information of the currently used configuration.
At the right-hand side there are some useless buttons which in future releases may have some function.

In the *Graph Tabs* and *Parameter Display Tabs* areas are two checkboxes:

![checkboxes](Pictures/HK_monitor_checkboxes.png)

These control what information is displayed by the parameter labels.

- *Engineering* When ticked engineering values are displayed in the parameter labels value field. When unticked the raw HK values are displayed.
**Note:** Only engineering values are plotted in the graphs.
- *Use Mib name* When ticked the parameter labels display the parameter name from the MIB. When unticked the *MIB Id* value is displayed as the parameter label.


### Operation <a name="operation"></a>

The TM TK Monitor program monitors a file and reads new data from the file when the file is updated. This file must exist before the TM HK Monitor program can read it.

**NOTE:** For the current version of the TM HK Monitor program the file to be read **must** exist and be zero bytes in size before being opened, otherwise it may not sync properly with the input file.

Before running the feeder program (see the next section, *Testing*) connect to the data file by pressing the **Start** button in the top left hand corner.

![start_button](Pictures/HK_monitor_start.png)

This will bring up a standard PyQt5 file open dialog box.
Navigate to where the zero byte-sized file to monitor is located and select it.
Once this is done the program is now monitoring the input file for changes and will update whenever the input file changes.
Now the feeder program can be started, see section *Testing*.


Once HK monitor program has connected to the input file the **Stop** button becomes enabled.
Pressing the **Stop** button closes the input file and stops monitoring.
To restart monitoring the feeder program must be stopped (see section *Testing*) and restarted before pressing the **Start** button again.

The **Quit** button can be pressed at any time.
This will bring up a dialog box confirming quit selection.
Cancelling the quit action will resume the HK monitor program.
If the quit option is selected then the feeder program must be manually stopped (see section *Testing*).

An image of the TM HK monitor receiving data follows:

![running](Pictures/hk_monitor_running.png)


### Testing <a name="testing"></a>

Run the feeder program in an xterm window:

`perl feed.pl <input file.csv> <output file.csv>`

or

`./feed.pl <input file.csv> <output file.csv>`

where *\<input file.csv>* is the name of an existing .csv DARTS dump file and *\<output file.csv>* is a file which will be written to.
An existing input file exists in the distribution directory, called *CEA\_Day3\_3.25.csv*.

**WARNING:**
The *\<output file.csv>* will be truncated to zero bytes before being written to so never use the file *CEA\_Day3\_3.25.csv* as the output file.

The feed.pl program will print a line to the screen showing it's parameters and then read the input file and write to the output file every second.
There will be no other output to the screen.
The output written to the output file will be the DARTS dump format.

To terminate the feed program press ctl-C in the xterm it is running in.
If this is done while the HK monitor program is still reading the *\<output file.csv>* then the HK monitor program will just wait in vain forever for changes.

The feeder program will automatically terminate when it reaches the end of the input file.


### Future Work <a name="future"></a>

- Add a method of loading a new configuration during run-time.
- Add a means for adding new tabs to the *Graph Tabs* and *Parameter Display Tabs* during run-time. These will be generated by the parameter chooser program.
- Add a method for opening new parameter display windows as generated by the parameter chooser program.
- Allow the graphs to plot raw values as well as engineering values.
- Modify the graphs to display time on the x-axis rather than just sample number.
- Modify the graphs to display event information.
- Add widgets to provide more user feedback about information received.
- IWS inter-program communication.