import sys

import json

from PyQt5.QtWidgets import QApplication, QWidget, QMainWindow, QLabel, QGridLayout, QPushButton, QVBoxLayout, QFrame
from PyQt5.QtWidgets import QMessageBox, QAction, QHBoxLayout, QTabWidget, QGroupBox, QRadioButton, QListWidget, QSpacerItem, QSizePolicy
from PyQt5.QtCore    import Qt, QRect
from PyQt5.QtGui     import QPalette, QColor, QFont

#import HkMonitorWindow
import iws_tmhk_monitor

class LauncherWindow(QWidget):
    def __init__(self, title, *args, **kwargs):
        super(LauncherWindow, self).__init__(*args, **kwargs)

        self.setWindowTitle(title)
        label = QLabel("This is a label")
        label.setAlignment(Qt.AlignCenter)
#        self.setCentralWidget(label)
        self.name = 'MainWindow'

        self.open_button = QPushButton("Launch HKTM Monitor")
        self.close_button = QPushButton("Close HKTM Monitor")
        self.open_button.setEnabled(False)
        self.close_button.setEnabled(False)
        self.open_button.clicked.connect(self.open_subwindow)
        self.close_button.clicked.connect(self.close_subwindow)

        self.windows = []
        self.wdict = {}

        self.win = 0

        self.subWindow = None

        self.configuration = 'packet_defaults'
        self.source = ''

        layout = QVBoxLayout()
        layout.addWidget(label)

        launch_box = QGroupBox()
        launch_box_layout = QVBoxLayout()

        radio_box = QGroupBox("Configuration")
        self.radio_packets = QRadioButton("Packets")
        self.radio_systems = QRadioButton("Systems")
        self.radio_packets.config = 'packet_defaults'
        self.radio_systems.config = 'system_defaults'
        self.radio_packets.toggled.connect(self.config_toggle)
        self.radio_systems.toggled.connect(self.config_toggle)
        self.radio_packets.setChecked(True)
        radio_layout = QGridLayout()
        radio_layout.addWidget(self.radio_packets, 0, 0)
        radio_layout.addWidget(self.radio_systems, 0, 1)
        radio_box.setLayout(radio_layout)

        data_box = QGroupBox("Data Source")
        self.listWidget = QListWidget()
        self.listWidget.insertItem(0,"~/Desktop/ex1.csv")
        self.listWidget.insertItem(1,"~/Desktop/foo.csv")
        self.listWidget.insertItem(2,"~/Desktop/bar.csv")
        self.listWidget.insertItem(3,"~/ex1.csv")
        self.listWidget.insertItem(4,"/media/pc_export/ex1.csv")
        self.listWidget.clicked.connect(self.list_clicked)

        self.delete_button = QPushButton("-")
        self.add_button = QPushButton("+")
        #spacer = QSpacerItem(5, 5, QSizePolicy.Ignored)
        spacer = QWidget()
        spacer.setSizePolicy(QSizePolicy.Ignored, QSizePolicy.Ignored)

        button_frame = QFrame()
        button_frame_layout = QHBoxLayout()
        #button_frame_layout.setSpacing(5)
        #button_frame_layout.setMargin(0)
        button_frame_layout.setContentsMargins(0, 0, 0, 0)
        button_frame_layout.addWidget(self.delete_button)
        button_frame_layout.addWidget(self.add_button)
        button_frame_layout.addWidget(spacer)

        button_frame.setLayout(button_frame_layout)

        data_layout = QVBoxLayout()
        data_layout.addWidget(self.listWidget)
        data_layout.addWidget(button_frame)

        data_box.setLayout(data_layout)

        launch_box_layout.addWidget(radio_box)
        launch_box_layout.addWidget(data_box)
        launch_box_layout.addWidget(self.open_button)
        launch_box_layout.addWidget(self.close_button)
        launch_box.setLayout(launch_box_layout)

        layout.addWidget(launch_box)

        self.setLayout(layout)

        quit = QAction("Quit", self)
        quit.triggered.connect(self.closeEvent)

    def config_toggle(self):
        button = self.sender()
        self.configuration = button.config
        print(self.configuration)

    def list_clicked(self):
        self.source = self.listWidget.currentItem().text()
        print("Data source is " + self.source)
        self.open_button.setEnabled(True)

    def open_subwindow(self):
        if self.subWindow == None:
            self.close_button.setEnabled(True)
            self.open_button.setEnabled(False)
            #s = HkMonitorWindow.HkMonitorWindow(self.configuration + ".json", self.source)
            #s = iws_tmhk_monitor.HkMonitorWindow(self.configuration + ".json", self.source)
            s = iws_tmhk_monitor.HkMonitorWindow(config=self.configuration + ".json", data_source=self.source)
            self.subWindow = s
            s.show()

    def close_subwindow(self):
        if self.subWindow != None:
            ok = self.subWindow.closeWindow()
            self.subWindow = None
            print("HkMonitor window close ok: " + str(ok))
            self.close_button.setEnabled(False)
            self.open_button.setEnabled(True)

if __name__ == '__main__':
    app = QApplication(sys.argv)
    #app.setStyleSheet("QGroupBox {background-color:pink;}")
    window = LauncherWindow("MainWindow")
    window.show()

    sys.exit(app.exec_())
