from PyQt5 import QtWidgets, QtGui, QtCore
from PyQt5.QtCore import Qt, QFileInfo
from PyQt5.QtWidgets import QFileDialog, QMessageBox

import re

import iws_defs
import chooser_defs

from lib.Lists.Lists import ListItem, ChildListItem, ParentListItem, List, MasterList

import lib.System.Euclid as Euc

#choices = ['system', 'packets']

#system_names = ['CDPU', 'OBS-G', 'PMCU', 'ROE', 'RPSU']
#system_tabs = {'CDPU'  : chooser_defs.cdpu_parameters,
#               'OBS-G' : chooser_defs.obsg_parameters,
#               'PMCU'  : chooser_defs.pmcu_parameters,
#               'ROE'   : chooser_defs.roe_parameters,
#               'RPSU'  : chooser_defs.rpsu_parameters}
#
#packet_names = ['RT_FAST', 'RT_SLOW', 'RT_SUPERFAST', 'ST_SLOW', 'ST_SUPERFAST']
#packet_tabs = {'RT_FAST'      : chooser_defs.rt_fast_parameters,
#               'RT_SLOW'      : chooser_defs.rt_slow_parameters,
#               'RT_SUPERFAST' : chooser_defs.rt_superfast_parameters,
#               'ST_SLOW'      : chooser_defs.st_slow_parameters,
#               'ST_SUPERFAST' : chooser_defs.st_superfast_parameters}


def create_tree_widget():
    tree = QtWidgets.QTreeWidget()
    tree.setMinimumSize(QtCore.QSize(iws_defs.TEXT_FIELD_MINIMUM_SIZE, 100))
    tree.setColumnCount(4)
    tree.setHeaderLabels(["Parameters", "MIB ID", "Source", "Unit"])
    return tree


def create_list_box_button_frame(owner = None):
    frame = QtWidgets.QFrame()
    layout = QtWidgets.QHBoxLayout()
    #self.delete_group_button = QtWidgets.QPushButton('Delete')
    #self.delete_group_button.clicked.connect(self.delete_group)
    delete_group_button = QtWidgets.QPushButton('Delete')
    delete_group_button.clicked.connect(owner.delete_group)
    #layout.addWidget(self.delete_group_button)
    layout.addWidget(delete_group_button)
    layout.addStretch(1)
    frame.setLayout(layout)
    return frame


def create_list_frame(owner = None):
    frame = QtWidgets.QFrame()
    frame.setStyleSheet("background-color:lightblue;")
    frame.setFrameStyle(QtWidgets.QFrame.StyledPanel)
    
    layout = QtWidgets.QVBoxLayout()
    layout.setContentsMargins(0, 0, 0, 0)
    
    """
    group_list_box = QtWidgets.QGroupBox('Groups:')
    group_list_box_layout = QtWidgets.QVBoxLayout()
    #group_list_box_layout.setContentsMargins(0, 0, 0, 0)
    #self.group_list_view = QtWidgets.QListView()
    group_list_box.setMinimumSize(QtCore.QSize(iws_defs.TEXT_FIELD_MINIMUM_SIZE, 100))
    #self.group_list_view = QtWidgets.QListWidget()
    self.group_list_view = List.List("Groups")
    self.group_list_view.setAcceptDrops(False)
    """
    """
    group_list_box_layout.addWidget(self.group_list_view)
    group_list_box.setLayout(group_list_box_layout)
    """
    
    list_box_button_frame = create_list_box_button_frame(owner = owner)
    
    #self.gp_tree = self.create_tree_widget()
    owner.gp_tree = create_tree_widget()
    
    #layout.addWidget(self.gp_tree)
    layout.addWidget(owner.gp_tree)
    
    layout.addWidget(list_box_button_frame)
    frame.setLayout(layout)
    return frame


def createList(label, parameter_file, type, main_tab_number):

    #list = MasterList.MasterList(label)
    list = List(label)
    list.resize(200,100)
    #list.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
    list.setAcceptDrops(False)
    
    seen_parent = False
    blank = re.compile('^$')
    comment = re.compile('^\?')
    alt_comment = re.compile('^;')
    minus = re.compile('^---')
    pluses = re.compile('^\+\+\+')
    
    try:
        with open(parameter_file, 'r') as f:
            #print(f"Getting items for {label}")
            sub_tab_number = 0
            row = 0
            for line in f:
                # txt
                #items = line.split()
                # csv
                items = line.rstrip('\n').split(',')
                print(f"{items}")
                if blank.match(line):
                    seen_parent = False
                    continue
                if comment.match(line):
                    continue
                if alt_comment.match(line):
                    continue
                if minus.match(line):
                    seen_parent = True
                    # ---,VIS_RPSU_TEMP_1,????,rt_slow,degC,0x012E,rpsu_temp1,No description
                    #print(f'ParentsListItem({items[1]}, {items[2]}, {items[3]}, {items[4]}, {row}')
                    parent = ParentListItem(items[1], items[2], items[3], items[4], row)
                    list.addItem(parent)
                    row += 1
                    continue
                if pluses.match(line):
                    seen_parent = True
                    # +++,VIS_ROE_FPGA_BIA_ID_T,VS1T2056,st_slow,degC,0x0166,fpga_bias_id_t,FPGA Bias ID_Top voltage
                    parent = ParentListItem(items[1], items[2], items[3], items[4], row)
                    list.addItem(parent)
                    row += 1
                    continue
                if seen_parent:
                ###child = ListItem.ListItem(items[0], items[1], items[2], items[3], row)
                    # VIS_RPSU_TEMP_1_01,VP1T2500,rt_slow,degC,,,
                    child = ChildListItem(items[0], items[1], items[2], items[3], row)
                    child.main_tab = main_tab_number
                    child.sub_tab = sub_tab_number
                    child.set_parent(parent.name)
                    child.set_parent_id(parent.id)
                    list.addItem(child)
                    #                        print(f"Adding child: {child.get_components()}, parent: {child.parent}")
                else:
                    ###orphan = ListItem.ListItem(items[0], items[1], items[2], items[3], row)
                    # VIS_RPSU_VAN_P_V01,VP1T2508,st_slow,V,0x0129,rpsu_van_p_v,RPSU_VAN_Pos_V_Mon
                    orphan = ChildListItem(items[0], items[1], items[2], items[3], row)
                    orphan.main_tab = main_tab_number
                    orphan.sub_tab = sub_tab_number
                    list.addItem(orphan)
                    seen_parent = False
                    #print(f"Adding child: {orphan.get_components()}")
                row += 1
            sub_tab_number += 1
    except Exception as e:
        print(f"EXCEPTION {e}")
    finally:
        #f.close()
        pass

    return list


def createTab(type, owner = None):
    font = QtGui.QFont()
    font.setPointSize(9)
    #font.setBold(True)
    font.setWeight(50)
    
    names = Euc.system_names if type == 'system' else Euc.packet_names
    tabs =  Euc.system_tabs  if type == 'system' else Euc.packet_tabs
    
    tabWidget = QtWidgets.QTabWidget()
    tabWidget.setFont(font)
    tabWidget.setObjectName("tabWidget")
    tab_number = 0
    for name in names:
        layout = QtWidgets.QHBoxLayout()
        tab = QtWidgets.QWidget()
        tab.setObjectName(name + '_tab')
        
        #list = owner.createList(name, tabs[name], type, tab_number)
        list = createList(name, tabs[name], type, tab_number)
        
        #self.parameter_trees[type][name].append(list)
        owner.parameter_trees[type][name] = list
        
        #self.parameter_trees[type][name] = self.createTree(tab, name, tabs[name], type)
        
        layout.addWidget(list, stretch = 1)
        tab.setLayout(layout)
        
        tabWidget.addTab(tab, name)
        tab_number += 1

    tabWidget.setCurrentIndex(0)
    return tabWidget


def createTabWidgetFrame(owner = None):
    frame = QtWidgets.QFrame()
    #frame.setStyleSheet("background-color:lightgreen;")
    frame.setFrameStyle(QtWidgets.QFrame.StyledPanel)
    
    #font = QtGui.QFont()
    #font.setPointSize(11)
    #font.setBold(True)
    #font.setWeight(75)
    
    owner.masterTabWidget = QtWidgets.QTabWidget()
    #self.masterTabWidget.setStyleSheet("background-color:lightgreen;")
    #self.masterTabWidget.setFont(font)
    for type in Euc.choices:
        #owner.masterTabWidget.addTab(owner.createTab(type), type)
        owner.masterTabWidget.addTab(createTab(type, owner), type)
        
    layout = QtWidgets.QVBoxLayout()
    layout.addWidget(owner.masterTabWidget)
        
    buttons_frame = QtWidgets.QFrame()
    buttons_frame_layout = QtWidgets.QHBoxLayout()
    buttons_frame_layout.setContentsMargins(0, 0, 0, 0)
    
    clearButton = QtWidgets.QPushButton("Clear")
    clearButton.clicked.connect(lambda:owner.clearAllClicked(clearButton))

    groupButton = QtWidgets.QPushButton("Group")
    groupButton.clicked.connect(lambda:owner.groupClicked(clearButton))
    
    buttons_frame_layout.addWidget(clearButton)
    buttons_frame_layout.addWidget(groupButton)
    buttons_frame_layout.addStretch(1)
    buttons_frame.setLayout(buttons_frame_layout)
    
    layout.addWidget(buttons_frame)

    frame.setLayout(layout)
    return frame
    

def create_control_frame(owner = None):
    frame = QtWidgets.QFrame()
    frame.setObjectName("control_frame")
    frame.setStyleSheet("background-color:yellow;")
    frame.setFrameStyle(QtWidgets.QFrame.StyledPanel)
    
    exportButton = QtWidgets.QPushButton("Export")
    exportButton.clicked.connect(owner.exportClicked)
    
    cancelButton = QtWidgets.QPushButton("Cancel")
    cancelButton.clicked.connect(owner.cancelClicked)
    
    quitButton = QtWidgets.QPushButton("Quit")
    quitButton.clicked.connect(owner.quitClicked)
    
    layout = QtWidgets.QHBoxLayout()
    layout.addWidget(cancelButton)
    layout.addWidget(exportButton)
    layout.addStretch(1)
    layout.addWidget(quitButton)
    
    frame.setLayout(layout)
    return frame




def create_top_frame(owner = None):
    frame = QtWidgets.QFrame()
    frame.setStyleSheet("background-color:lightgreen;")
    
    layout = QtWidgets.QHBoxLayout()
    layout.setContentsMargins(0, 0, 0, 0)
    
    ###list_frame = self.create_list_frame()
    list_frame = create_list_frame(owner = owner)
    
    owner.tabWidgetFrame = createTabWidgetFrame(owner = owner)
    #self.tabWidgetFrame.setMinimumSize(QtCore.QSize(210, 100))
    owner.tabWidgetFrame.setMinimumSize(QtCore.QSize(iws_defs.TEXT_FIELD_MINIMUM_SIZE + 140, 100))
    
    layout.addWidget(owner.tabWidgetFrame, stretch = 1)
    layout.addWidget(list_frame)
    
    frame.setLayout(layout)
    return frame


def create_main_layout(owner = None):
    layout = QtWidgets.QVBoxLayout()
    
    top_frame     = create_top_frame(owner = owner)
    control_frame = create_control_frame(owner = owner)

    layout.addWidget(top_frame)
    layout.addWidget(control_frame, stretch = 0)

    return layout


